@@ -1,13 +1,13 @@
-# $Id: Extrefs.pm,v 1.10 2004/07/16 14:44:56 rcaputo Exp $
+# $Id: Extrefs.pm 2087 2006-09-01 10:24:43Z bsmith $
 
 # The data necessary to manage tagged extra/external reference counts
 # on sessions, and the accessors to get at them sanely from other
 # files.
 
-package POE::Resources::Extrefs;
+package POE::Resource::Extrefs;
 
 use vars qw($VERSION);
-$VERSION = do {my@r=(q$Revision: 1.10 $=~/\d+/g);sprintf"%d."."%04d"x$#r,@r};
+$VERSION = do {my($r)=(q$Revision: 2087 $=~/(\d+)/);sprintf"1.%04d",$r};
 
 # These methods are folded into POE::Kernel;
 package POE::Kernel;
@@ -167,7 +167,7 @@ __END__
 
 =head1 NAME
 
-POE::Resources::Extrefs - tagged "extra" ref. count management for POE::Kernel
+POE::Resource::Extrefs - tagged "extra" ref. count management for POE::Kernel
 
 =head1 SYNOPSIS
 
