@@ -1,4 +1,4 @@
-# $Id: Signals.pm 2126 2006-09-16 05:33:53Z rcaputo $
+# $Id: Signals.pm 2192 2007-07-04 21:31:15Z rcaputo $
 
 # The data necessary to manage signals, and the accessors to get at
 # that data in a sane fashion.
@@ -6,7 +6,7 @@
 package POE::Resource::Signals;
 
 use vars qw($VERSION);
-$VERSION = do {my($r)=(q$Revision: 2126 $=~/(\d+)/);sprintf"1.%04d",$r};
+$VERSION = do {my($r)=(q$Revision: 2192 $=~/(\d+)/);sprintf"1.%04d",$r};
 
 # These methods are folded into POE::Kernel;
 package POE::Kernel;
@@ -62,10 +62,7 @@ use vars (
 # A flag to tell whether we're currently polling for signals.
 my $polling_for_signals = 0;
 
-# A flag determining whether there are child processes.  Starts true
-# so our waitpid() loop can run at least once.  Starts false when
-# running in an Apache handler so our SIGCHLD hijinx don't interfere
-# with the web server.
+# A flag determining whether there are child processes.
 my $kr_child_procs = exists($INC{'Apache.pm'}) ? 0 : 1;
 
 sub _data_sig_preload {
@@ -105,6 +102,11 @@ my %_safe_signals;
 sub _data_sig_initialize {
   my $self = shift;
 
+  # Initialize this to a true value so our waitpid() loop can run at
+  # least once.  Starts false when running in an Apache handler so our
+  # SIGCHLD hijinx don't interfere with the web server.
+  $kr_child_procs = exists($INC{'Apache.pm'}) ? 0 : 1;
+
   # In case we're called multiple times.
   unless (keys %_safe_signals) {
     foreach my $signal (keys %SIG) {
@@ -314,6 +316,14 @@ sub _data_sig_pids_ses {
   return scalar keys %{$kr_sessions_to_pids{$session}};
 }
 
+sub _data_sig_pids_is_ses_watching {
+  my ($self, $session, $pid) = @_;
+  return(
+    exists($kr_sessions_to_pids{$session}) &&
+    exists($kr_sessions_to_pids{$session}{$pid})
+  );
+}
+
 ### Return a signal's type, or SIGTYPE_BENIGN if it's not special.
 
 sub _data_sig_type {
